package sun.reflect;

import java.lang.reflect.Modifier;
import java.lang.reflect.Field;
import sun.misc.Unsafe;

abstract class UnsafeFieldAccessorImpl extends FieldAccessorImpl
{
    static final Unsafe unsafe;
    protected final Field field;
    protected final long fieldOffset;
    protected final boolean isFinal;
    
    UnsafeFieldAccessorImpl(Field f) {
        this.field = f;
        if (Modifier.isStatic(f.getModifiers())) {
            this.fieldOffset = UnsafeFieldAccessorImpl.unsafe.staticFieldOffset(f);
        }
        else {
            this.fieldOffset = UnsafeFieldAccessorImpl.unsafe.objectFieldOffset(f);
        }
        this.isFinal = Modifier.isFinal(f.getModifiers());
    }
    
    protected void ensureObj(Object o) {
        if (!this.field.getDeclaringClass().isAssignableFrom(o.getClass())) {
            this.throwSetIllegalArgumentException(o);
        }
    }
    
    private String getQualifiedFieldName() {
        return this.field.getDeclaringClass().getName() + "." + this.field.getName();
    }
    
    protected IllegalArgumentException newGetIllegalArgumentException(String str) {
        return new IllegalArgumentException("Attempt to get " + this.field.getType().getName() + " field \"" + this.getQualifiedFieldName() + "\" with illegal data type conversion to " + str);
    }
    
    protected void throwFinalFieldIllegalAccessException(String s, String s2) throws IllegalAccessException {
        throw new IllegalAccessException(this.getSetMessage(s, s2));
    }
    
    protected void throwFinalFieldIllegalAccessException(Object o) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException((o != null) ? o.getClass().getName() : "", "");
    }
    
    protected void throwFinalFieldIllegalAccessException(boolean b) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("boolean", Boolean.toString(b));
    }
    
    protected void throwFinalFieldIllegalAccessException(char c) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("char", Character.toString(c));
    }
    
    protected void throwFinalFieldIllegalAccessException(byte b) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("byte", Byte.toString(b));
    }
    
    protected void throwFinalFieldIllegalAccessException(short s) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("short", Short.toString(s));
    }
    
    protected void throwFinalFieldIllegalAccessException(int i) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("int", Integer.toString(i));
    }
    
    protected void throwFinalFieldIllegalAccessException(long i) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("long", Long.toString(i));
    }
    
    protected void throwFinalFieldIllegalAccessException(float f) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("float", Float.toString(f));
    }
    
    protected void throwFinalFieldIllegalAccessException(double d) throws IllegalAccessException {
        this.throwFinalFieldIllegalAccessException("double", Double.toString(d));
    }
    
    protected IllegalArgumentException newGetBooleanIllegalArgumentException() {
        return this.newGetIllegalArgumentException("boolean");
    }
    
    protected IllegalArgumentException newGetByteIllegalArgumentException() {
        return this.newGetIllegalArgumentException("byte");
    }
    
    protected IllegalArgumentException newGetCharIllegalArgumentException() {
        return this.newGetIllegalArgumentException("char");
    }
    
    protected IllegalArgumentException newGetShortIllegalArgumentException() {
        return this.newGetIllegalArgumentException("short");
    }
    
    protected IllegalArgumentException newGetIntIllegalArgumentException() {
        return this.newGetIllegalArgumentException("int");
    }
    
    protected IllegalArgumentException newGetLongIllegalArgumentException() {
        return this.newGetIllegalArgumentException("long");
    }
    
    protected IllegalArgumentException newGetFloatIllegalArgumentException() {
        return this.newGetIllegalArgumentException("float");
    }
    
    protected IllegalArgumentException newGetDoubleIllegalArgumentException() {
        return this.newGetIllegalArgumentException("double");
    }
    
    protected String getSetMessage(String s, String str) {
        String str2 = "Can not set";
        if (Modifier.isStatic(this.field.getModifiers())) {
            str2 += " static";
        }
        if (this.isFinal) {
            str2 += " final";
        }
        String string = str2 + " " + this.field.getType().getName() + " field " + this.getQualifiedFieldName() + " to ";
        String s2;
        if (str.length() > 0) {
            s2 = string + "(" + s + ")" + str;
        }
        else if (s.length() > 0) {
            s2 = string + s;
        }
        else {
            s2 = string + "null value";
        }
        return s2;
    }
    
    protected void throwSetIllegalArgumentException(String s, String s2) {
        throw new IllegalArgumentException(this.getSetMessage(s, s2));
    }
    
    protected void throwSetIllegalArgumentException(Object o) {
        this.throwSetIllegalArgumentException((o != null) ? o.getClass().getName() : "", "");
    }
    
    protected void throwSetIllegalArgumentException(boolean b) {
        this.throwSetIllegalArgumentException("boolean", Boolean.toString(b));
    }
    
    protected void throwSetIllegalArgumentException(byte b) {
        this.throwSetIllegalArgumentException("byte", Byte.toString(b));
    }
    
    protected void throwSetIllegalArgumentException(char c) {
        this.throwSetIllegalArgumentException("char", Character.toString(c));
    }
    
    protected void throwSetIllegalArgumentException(short s) {
        this.throwSetIllegalArgumentException("short", Short.toString(s));
    }
    
    protected void throwSetIllegalArgumentException(int i) {
        this.throwSetIllegalArgumentException("int", Integer.toString(i));
    }
    
    protected void throwSetIllegalArgumentException(long i) {
        this.throwSetIllegalArgumentException("long", Long.toString(i));
    }
    
    protected void throwSetIllegalArgumentException(float f) {
        this.throwSetIllegalArgumentException("float", Float.toString(f));
    }
    
    protected void throwSetIllegalArgumentException(double d) {
        this.throwSetIllegalArgumentException("double", Double.toString(d));
    }
    
    static {
        unsafe = Unsafe.getUnsafe();
    }
}
